#####
##
# OrderedObjectManager / maik.jablonski@uni-bielefeld.de
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# This product uses slightly modified codes from the OrderedFolder-Product
# written by Stephan Richter / copyrighted by iuveno AG
# Please read the LICENSE.txt!
##
####

from Globals import ImageFile,DTMLFile
from OFS.ObjectManager import ObjectManager
from types import StringType

ObjectManager.manage_main = DTMLFile('dtml/manage_main', globals())

misc_ = {'up'     : ImageFile('www/up.gif', globals()),
         'down'   : ImageFile('www/down.gif', globals()),
         'top'    : ImageFile('www/top.gif', globals()),
         'bottom' : ImageFile('www/bottom.gif', globals())}

def get_object_position(self, id):
	"""Get the position of a particular object. This allows
	   to generate URLs in the application."""

	ids = map(lambda obj: obj['id'], self._objects)
	if id in ids:
	    return ids.index(id)
	else:
	    # If the object was not found, throw an error.
	    # Yeah, that is good raise 'ObjectNotFound',
	    # 'The object with the id \'%s\' does not exist.' %id
	    raise 'ObjectNotFound', self.aq_parent._objects

ObjectManager.get_object_position=get_object_position    
    
def move_objects_by_positions(self, ids, position_delta):
        """ """

        if type(ids) is StringType:
            ids = (ids,)

        # Interestingly enough, we need to switch the order when
        # moving doen, so that the movements won't cancel each
        # other
        if position_delta > 0:
            ids = list(ids)
            ids.reverse()
            
        moved_objects = 0
        
        for id in ids:
            old_position = self.get_object_position(id)
            new_position = old_position + position_delta
            object = self._objects[old_position]
            # Make sure the new position makes sense and is valid
            if not (old_position == new_position  or
                    new_position > len(self._objects) or
                    new_position < 0):
                
                objects = list(self._objects)
                # now delete the entry ...
                objects = filter(lambda obj, id=id: obj['id'] != id, objects)
                # ... and now add it again
                objects.insert(new_position, object)
                self._objects = tuple(objects)

                moved_objects += 1

        return moved_objects

ObjectManager.move_objects_by_positions=move_objects_by_positions

def move_objects_to_position(self, ids, position):
        """Moves a set of objects to a particular position. The
           original position is the

           Deprecated. Use move_objects_by_positions instead. 
           """
        if type(ids) is StringType:
            ids = (ids,)
        return self.move_objects_by_positions(ids,
                   position - self.get_object_position(ids[0]))

ObjectManager.move_objects_to_position=move_objects_to_position

def move_objects_up(self, ids):
        """the nuts-and-bolts version of moving the items up"""
        return self.move_objects_by_positions(ids, -1)

ObjectManager.move_objects_up=move_objects_up

def move_objects_down(self, ids):
        """the nuts-and-bolts version of moving the items down"""
        return self.move_objects_by_positions(ids, +1)

ObjectManager.move_objects_down=move_objects_down

def move_objects_to_top(self, ids):
        """the nuts-and-bolts version of moving the items to the top"""
        if type(ids) is StringType:
            ids = (ids,)

        position_delta = - self.get_object_position(ids[0])
        return self.move_objects_by_positions(ids, position_delta)

ObjectManager.move_objects_to_top=move_objects_to_top

def move_objects_to_bottom(self, ids):
        """the nuts-and-bolts version of moving the items to the bottom"""
        if type(ids) is StringType:
            ids = (ids,)

        # Whee, we will do the reverse twice, but for going to the bottom
        # there is no other choice.
        ids = list(ids)
        ids.reverse()

        position_delta = len(self.objectIds()) - self.get_object_position(ids[-1])
        return self.move_objects_by_positions(ids, position_delta)

ObjectManager.move_objects_to_bottom=move_objects_to_bottom

def manage_move_objects_up(self, ids=None, REQUEST=None):
        """moves selected items up by one, if possible, in the
        folder; returns HTML or integer confirmation"""
        if type(ids) is StringType:
            ids = (ids,)
            
        if not ids:
           attempt=0
	else:
           attempt = self.move_objects_up(ids)

        if REQUEST:
            REQUEST.RESPONSE.redirect('manage_main')
        return attempt

ObjectManager.manage_move_objects_up=manage_move_objects_up

def manage_move_objects_down(self, ids=None, REQUEST=None):
        """moves selected items down by one, if possible, in the
        folder; returns HTML or integer confirmation"""
        if type(ids) is StringType:
            ids = (ids,)

        if not ids:
            attempt=0
	else:
            attempt = self.move_objects_down(ids)

        if REQUEST:
            REQUEST.RESPONSE.redirect('manage_main')
        return attempt

ObjectManager.manage_move_objects_down=manage_move_objects_down

def manage_move_objects_to_top(self, ids=None, REQUEST=None):
        """moves selected items to top of folder; returns HTML or
        integer confirmation"""
        if type(ids) is StringType:
            ids = (ids,)

        if not ids:
            attempt=0
	else:            
            attempt=self.move_objects_to_top(ids)

        if REQUEST:
            REQUEST.RESPONSE.redirect('manage_main')
        return attempt

ObjectManager.manage_move_objects_to_top=manage_move_objects_to_top

def manage_move_objects_to_bottom(self, ids=None, REQUEST=None):
        """moves selected items to bottom of folder; returns HTML or
        integer confirmation"""
        if type(ids) is StringType:
            ids = (ids,)

        if not ids:
            attempt=0
	else:
            attempt = self.move_objects_to_bottom(ids)

        if REQUEST:
            REQUEST.RESPONSE.redirect('manage_main')
        return attempt

ObjectManager.manage_move_objects_to_bottom=manage_move_objects_to_bottom

_old_manage_renameObject = ObjectManager.inheritedAttribute('manage_renameObject')
def manage_renameObject(self, id, new_id, REQUEST=None):
        """Rename a particular sub-object"""
        # Since manage_renameObject uses _setObject manually, we have to
        # take care of the order after it is done. Geez, I guess it is
        # not possible to do it differently. PLEASE submit a patch, if
        # you can come up with something better!
        position = self.get_object_position(id)
        result = self._old_manage_renameObject(id, new_id, REQUEST)
        self.move_objects_to_position(new_id, position)

        return result

_old_setObject = ObjectManager._setObject
def _setObject(self,id,object,roles=None,user=None, set_owner=1, position=None):
        result = self._old_setObject(id, object, roles, user, set_owner)

        # Now, let's get the position right.
        if position is None:
            position = len(self._objects)
        self.move_objects_to_position(id, position)

        return result


def manage_orderObjects(self, ids=None, REQUEST=None):
	""" Orders a container with the ordered list 'ids' """
		
	if ids:
		for id in ids:
        	    old_position = self.get_object_position(id)
	            new_position = ids.index(id)
        	    object = self._objects[old_position]
	            # Make sure the new position makes sense and is valid
	            if not (old_position == new_position  or
        	            new_position > len(self._objects) or
                	    new_position < 0):
                
	                objects = list(self._objects)
        	        # now delete the entry ...
                	objects = filter(lambda obj, id=id: obj['id'] != id, objects)
	                # ... and now add it again
        	        objects.insert(new_position, object)
	                self._objects = tuple(objects)

	if REQUEST:
		REQUEST.RESPONSE.redirect('manage_main')
	    
ObjectManager.manage_orderObjects = manage_orderObjects
